import argparse
from typing import List, Dict, Callable, Union

from blueness.argparse.generic import sys_exit

from blue_options.logger import logger


def get_help(
    tokens: List[str],
    help_functions: Union[Callable, Dict[str, Union[Callable, Dict]]],
    mono: bool,
) -> str:
    if callable(help_functions):
        return help_functions(tokens, mono=mono)

    if not tokens:
        thing = help_functions
    elif tokens[0] in help_functions:
        return get_help(
            tokens[1:],
            help_functions[tokens[0]],
            mono=mono,
        )
    else:
        return ""

    if isinstance(thing, dict):
        return "\n".join(
            [
                get_help(
                    tokens[1:],
                    item,
                    mono=mono,
                )
                for item in thing.values()
            ]
        )

    assert callable(thing)
    return thing(tokens[1:], mono=mono)


def help_main(
    NAME: str,
    help_functions: Union[Callable, Dict[str, Union[Callable, Dict]]],
):
    parser = argparse.ArgumentParser(NAME)
    parser.add_argument(
        "--command",
        type=str,
    )
    parser.add_argument(
        "--mono",
        type=int,
        default=0,
        help="0|1",
    )
    args = parser.parse_args()

    content = get_help(
        tokens=args.command.strip().split(" "),
        help_functions=help_functions,
        mono=args.mono == 1,
    )

    if content:
        print(content)
    else:
        logger.warning(f'"{args.command}": help not found.')

    sys_exit(None, NAME, "help", bool(content))



from functools import reduce
from typing import List, Dict, Tuple, Union

from blue_options import env


def error(
    message: str,
    mono: bool = False,
) -> str:
    RED = "" if mono else env.RED
    NC = "" if mono else env.NC

    return f"❗️ {RED}{message}{NC}\n"


def hr(
    width: int,
    pattern: str,
    mono: bool = False,
) -> str:
    CYAN = "" if mono else env.CYAN
    NC = "" if mono else env.NC

    return "{}{}{}".format(
        CYAN,
        ((width // len(pattern) + 1) * pattern)[:width],
        NC,
    )


def show_usage(
    command: Union[List[str], str],
    description: str,
    details: Dict[str, List[str]] = {},
    mono: bool = False,
) -> str:
    NC = "" if mono else env.NC
    LIGHTBLUE = "" if mono else env.LIGHTBLUE
    CYAN = "" if mono else env.CYAN

    if isinstance(command, list):
        command = env.ABCUL.join(command)

    return "\n".join(
        [
            f"{LIGHTBLUE}{command}{NC}",
            f"{CYAN} . {description}{NC}",
        ]
        + reduce(
            lambda x, y: x + y,
            [
                (
                    [f"{CYAN}   {detail}{NC}"]
                    + (
                        [
                            f"{CYAN}      {item}{NC}"
                            for item in (
                                list_of_items
                                if isinstance(list_of_items, list)
                                else [list_of_items]
                            )
                        ]
                        if list_of_items
                        else []
                    )
                )
                for detail, list_of_items in details.items()
            ],
            [],
        )
    )


def xtra(
    options: str,
    mono: bool = False,
) -> str:
    EOP = "" if mono else env.EOP
    LIGHTBLUE = "" if mono else env.LIGHTBLUE

    return f"{EOP}{options}{LIGHTBLUE}"
